package com.zoniot.ccrc.security;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.commom.utils.JwtTokenUtil;
import com.zoniot.ccrc.commom.utils.RedisUtil;
import com.zoniot.ccrc.commom.utils.Util;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.entity.LoginLogs;
import com.zoniot.ccrc.entity.User;
import com.zoniot.ccrc.security.smsCode.SmsCodeException;
import com.zoniot.ccrc.service.LoginLogsService;
import com.zoniot.ccrc.service.TokenService;
import com.zoniot.ccrc.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.*;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.access.AccessDeniedHandler;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.logout.LogoutSuccessHandler;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;

/**
 * spring security处理器
 */
@Slf4j
@Configuration
public class SecurityHandlerConfig {

    private AjaxMessage ajaxMessage = new AjaxMessage();
    @Autowired
    JwtTokenUtil jwtTokenUtil;
    @Autowired
    private TokenService tokenService;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    private UserService userService;
    @Autowired
    private LoginLogsService loginLogsService;

    /**
     * 登陆成功，返回Token
     *
     * @return
     */
    @Bean(name = "userAuthenticationSuccessHandler")
    public AuthenticationSuccessHandler userAuthenticationSuccessHandler() {
        return new AuthenticationSuccessHandler() {

            @Override
            public void onAuthenticationSuccess(HttpServletRequest request, HttpServletResponse response,
                                                Authentication authentication) throws IOException, ServletException {
                response.setContentType("application/json;charset=UTF-8");
                String random = request.getParameter("random");
                String openId = request.getParameter("openId");



                ObjectMapper objectMapper = new ObjectMapper();
                LoginUser loginUser = (LoginUser) authentication.getPrincipal();
                String jwtToken = tokenService.saveToken(loginUser);
                ajaxMessage.setMsg(ResultStatus.OK, jwtToken);

                //清除验证码
                if (random != null) {
                    String key = "validateCode:" + random;
                    redisUtil.del(key.getBytes());
                }


                //绑定openId
                if (openId != null) {
                    log.info("login openId:"+ openId);
                    if (loginUser.getIsSuperAdmin() != 1 && !StringUtils.equals(openId, "") && StringUtils.equals(loginUser.getOpenid(), "")) {
                        //保存openid
                        User user = userService.findUserByOpenid(openId);
                        if (user == null) {
                            User userUpdate = new User();
                            userUpdate.setId(loginUser.getId());
                            userUpdate.setOpenid(openId);
                            userUpdate.setUpdateBy(loginUser.getUsername());
                            userUpdate.setUpdateDate(LocalDateTime.now());
                            userService.update(userUpdate);
                        }
                    }
                }

                //登陆日志
                Integer siteId = -1;
                if (loginUser != null && loginUser.getIsSuperAdmin() != 1) {
                    if (loginUser.getSiteId() != null) {
                        siteId = loginUser.getSiteId();
                    }
                }
                loginLogsService.AsyncInsertLoginLogs(new LoginLogs(null, siteId, loginUser.getId(), "无", Util.getIpAddr(request), LocalDateTime.now()));

                response.getWriter().write(objectMapper.writeValueAsString(ajaxMessage));


            }
        };
    }

    /**
     * 登陆失败
     *
     * @return
     */
    @Bean(name = "userAuthenticationFailureHandler")
    public AuthenticationFailureHandler userAuthenticationFailureHandler() {
        return new AuthenticationFailureHandler() {

            @Override
            public void onAuthenticationFailure(HttpServletRequest request, HttpServletResponse response,
                                                AuthenticationException exception) throws IOException, ServletException {
                response.setContentType("application/json;charset=UTF-8");
                ObjectMapper objectMapper = new ObjectMapper();
                if (exception instanceof BadCredentialsException ||
                        exception instanceof UsernameNotFoundException) {
                    ajaxMessage.setMsg(ResultStatus.USERNAME_NOT_FOUND_ERROR);
                } else if (exception instanceof LockedException) {
                    ajaxMessage.setMsg(ResultStatus.ACCOUNT_LOCKED_ERROR);
                } else if (exception instanceof CredentialsExpiredException) {
                    ajaxMessage.setMsg(ResultStatus.CREDENTIALS_EXPIRED_ERROR);
                } else if (exception instanceof AccountExpiredException) {
                    ajaxMessage.setMsg(ResultStatus.ACCOUNT_EXPIRED_ERROR);
                } else if (exception instanceof DisabledException) {
                    ajaxMessage.setMsg(ResultStatus.ACCOUNT_DISABLED_ERROR);
                } else if (exception instanceof SmsCodeException) {
                    ajaxMessage.setMsg(-101, exception.getMessage());
                } else {
                    ajaxMessage.setMsg(-1, exception.getMessage());

                }
                response.getWriter().write(objectMapper.writeValueAsString(ajaxMessage));
            }
        };

    }

    /**
     * 未登录，返回401
     *
     * @return
     */
    @Bean(name = "userAuthenticationEntryPoint")
    public AuthenticationEntryPoint userAuthenticationEntryPoint() {
        return new AuthenticationEntryPoint() {

            @Override
            public void commence(HttpServletRequest request, HttpServletResponse response,
                                 AuthenticationException authException) throws IOException, ServletException {
                response.setContentType("application/json;charset=UTF-8");
                ObjectMapper objectMapper = new ObjectMapper();
                ajaxMessage.setMsg(ResultStatus.ACCESS_UNVERIFIED_ERROR);
                response.getWriter().write(objectMapper.writeValueAsString(ajaxMessage));
            }
        };
    }

    /**
     * 退出处理
     *
     * @return
     */
    @Bean("userLogoutSuccessHandler")
    public LogoutSuccessHandler userLogoutSuccessHandler() {
        return new LogoutSuccessHandler() {

            @Override
            public void onLogoutSuccess(HttpServletRequest request, HttpServletResponse response,
                                        Authentication authentication) throws IOException, ServletException {
                response.setContentType("application/json;charset=UTF-8");
                ObjectMapper objectMapper = new ObjectMapper();
                ajaxMessage.setMsg(ResultStatus.OK);
                response.getWriter().write(objectMapper.writeValueAsString(ajaxMessage));
            }
        };

    }

    /**
     * 权限不足处理
     *
     * @return
     */
    @Bean(name = "userAccessDeniedHandler")
    public AccessDeniedHandler userAccessDeniedHandler() {
        return new AccessDeniedHandler() {
            @Override
            public void handle(HttpServletRequest request, HttpServletResponse response, AccessDeniedException accessDeniedException) throws IOException, ServletException {
                response.setContentType("application/json;charset=UTF-8");
                ObjectMapper objectMapper = new ObjectMapper();
                ajaxMessage.setMsg(ResultStatus.ACCESS_DENIED_ERROR);
                response.getWriter().write(objectMapper.writeValueAsString(ajaxMessage));
            }
        };

    }

}
